import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map.Entry;

/**
 * seperate from invertedindex class
 * to perform search method from inverted index
 */
public class Search {
	
	private Invertedindex inverted;
	
	public Search() {
		inverted = Invertedindex.getInvertedindex();
	}

	// Set a double object for score value to keep increasing
	public class doubleObj {
		
		private double scoreValue;
		
		// Constructor
		public doubleObj() {
			this.scoreValue = 0d;
		}
		
		// Increasing score
		public void increment(double scoreValue) {
			this.scoreValue += scoreValue;
		}
		
		// Return double not double object
		public double getDouble() {
			return this.scoreValue;
		}
	}
	
	// Set each element of Array list in query result has two components (ID of file and score)
	public class DocLevel {
		
		// file ID
		Integer ID;
		
		// Score value of this file
		double score;
		
		// Constructor 
		public DocLevel(double score, Integer ID) {
			this.ID = ID;
			this.score = score;
		}
	}
	
	// Compare method for score to sort the file
	private Comparator sortComparator() {
		return new Comparator() {
			public int compare(Object obj1, Object obj2) {
				
				Double double1 = ((DocLevel) obj1).score;
				Double double2 = ((DocLevel) obj2).score;
				
				if (double1 < double2) {
					return 1;
				}
				else if (double1 > double2) {
					return -1;
				}
				else {
					return 0;
				}
			}
		};
	}
	
	//Search word in the inverted index
	public ArrayList<DocLevel> search(String queryword) {
		
		inverted.lock.getReader();
		
		/*
		 * templist : Store search result into a temporary hashmap
		 * Integer : file ID
		 * Double Object : score
		 */
		HashMap<Integer, doubleObj> templist = new HashMap<Integer, doubleObj>(); 
		
		double avgLength = 0d;
		double sumLength = 0d;
		
		for (LinkInfo info : inverted.linkdata.getResult()) {
			sumLength += info.linklength;
		}
		avgLength = sumLength / inverted.linkdata.getResult().size();
		
		// Initialize the score function
		Score.BM25 scoreFunction = (new Score()).new BM25(avgLength, inverted.linkdata.getResult().size());
		
		// Initialize the value
		int num_docTerm = 0;
		
		// split query word with white space
		for (String str : queryword.split(" ")) {
			
			// remove non-alphabet word
			str = str.replaceAll("[^A-Za-z0-9]", "").toLowerCase();

			// The query word does not find out in this file
			if (inverted.datamap.get(str) == null) {
				
				// No terms in this document
				num_docTerm = 0;
			}
			
			// The query word found in the file 
			else {
				
				// set value of "number of terms in the document"
				num_docTerm = inverted.datamap.get(str).size();
				
				// Use the iterator to go though the hashmap
				for (Entry<Integer, ArrayList<Integer>> pointer : inverted.datamap.get(str).entrySet()) {
					
					// Initialize the value first in the temporary hashmap
					if (templist.containsKey(pointer.getKey()) == false) {
						doubleObj obj = new doubleObj();
						templist.put(pointer.getKey(), obj);
					}
					
					// Call score class to calculate the score 
					double score = scoreFunction.getScore(inverted.linkdata.getResult().get(pointer.getKey()).linklength, pointer.getValue().size(), num_docTerm);

					// Once the term has been found before, increment the score 
					templist.get(pointer.getKey()).increment(score);
				}
			}
		}
		
		/*
		 * Copy the hashmap into array list
		 * it is much more convenient for sorting
		 */
		ArrayList<DocLevel> sortlist = new ArrayList<DocLevel>();
		for (Entry<Integer, doubleObj> index : templist.entrySet()) {
			DocLevel doc = new DocLevel(index.getValue().getDouble(), index.getKey());
			sortlist.add(doc);
		}
		
		//Sort the score	
		Collections.sort(sortlist, sortComparator());
		
		inverted.lock.releaseReader();
		
		return sortlist;
	}
	
}
