import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * stripped useless tag or comment from html page
 */
public class TagStripper {
	
	/**
	 * an array list for html links
	 */
	public ArrayList<String> linklist;
	
	/**
	 * a stringbuffer for html page
	 */
	public StringBuffer bufHtml;
	
	public String title = "";
	
	public String relative = "";
	
	/**
	 * instance of RegexLibrary class
	 */
	private RegexLibrary regex = new RegexLibrary();
	
	public TagStripper(StringBuffer bufHtml) {
		this.bufHtml = bufHtml;
	}
	
	/**
	 * Start stripped classes
	 */
	public void stripStart() {
		
		this.getTitle();
		this.getRelative();
		this.stripJS();
		this.stripCss();
		this.strippedComments();
		this.stripTag();

		if (this.relative.equals("")) {

			if (this.bufHtml.length() < 100) {
				this.relative = this.bufHtml.toString();
			}
			else {
				this.relative = this.bufHtml.substring(0, 100) + " ...";
			}
		}
		
	}
	
	/**
	 * stripped each input regex with replace pattern
	 */
	private StringBuffer stripped(String regexPattern, String replacePattern) {
		
		StringBuffer buffer = new StringBuffer();
		
		// Including line terminator
		Pattern p = Pattern.compile(regexPattern, Pattern.DOTALL);
		Matcher m = p.matcher(this.bufHtml);
		
		/**
		 * Replace matched string with replacePattern
		 */
		while (m.find()) {
			m.appendReplacement(buffer, replacePattern);
		}
		
		if (buffer.length() != 0) {
			return m.appendTail(buffer);
		}
		// No matched items, return original string buffer
		else {
			return this.bufHtml;
		}
	}
	
	public void strippedComments() {
		String replacePattern = "";
		this.bufHtml = this.stripped(regex.regexComments, replacePattern);
	}
	
	/**
	 * Stripped <script>...</script> tags
	 */
	public void stripJS() {
		String replacePattern = "";
		this.bufHtml = this.stripped(regex.regexJS, replacePattern);
	}
	
	/**
	 * Stripped <style>...</style> tags
	 */
	public void stripCss() {
		String replacePattern = "";
		this.bufHtml = this.stripped(regex.regexCss, replacePattern);
	}
	
	/**
	 * Remove all left tags
	 */
	public void stripTag() {
		String replacePattern = " ";
		this.bufHtml = this.stripped(regex.regexTagComplete, replacePattern);
	}
	
	/**
	 * To check whether the url is valid or not
	 * Start with "http"
	 * can not end with pdf, jpg, png, css
	 */
	public boolean isURLvalid(String url) {
		if (url.equals("")) {
			return false;
		}
		if (!url.startsWith("http://")) {
			return false;
		}
		Pattern pURL = Pattern.compile(regex.linkException);
		Matcher mURL = pURL.matcher(url);
		return (!mURL.find());
	}
	
	/**
	 * Find links and save links into array list
	 */
	public void getLink() {
		
		String links = "";
		ArrayList<String> htmllinks = new ArrayList<String>();

		// Find <a>...</a> this format first 
		Pattern p = Pattern.compile(regex.regexLinks);
		Matcher m = p.matcher(this.bufHtml);
		
		// Then find href = "..." 
		Pattern phref = Pattern.compile(regex.regexHref);
		
		// If found <a>...</a>
		while(m.find()) {
			
			Matcher mhref = phref.matcher(m.group());
			
			while (mhref.find()) {
				links = mhref.group().replaceFirst(regex.hrefRemove, "").replaceFirst(regex.quoteRemove, "");
				if (isURLvalid(links)) {
					htmllinks.add(links);
				}
			}
		}
		this.linklist = htmllinks;
	}
	
	private void getTitle() {
		Pattern p = Pattern.compile(regex.regexTitle);
		Matcher m = p.matcher(this.bufHtml);
		if (m.find()) {
			this.title = m.group().replaceFirst("<title>", "").replaceFirst("</title>", "");
		}
	}

	private String setRelative(String str) {
		
		Pattern pResult = Pattern.compile(regex.regexFullFragment);
		Matcher mResult = pResult.matcher(str);
		String result = "";
		if (mResult.find()) {
			result = mResult.group().replaceFirst(regex.regexRemoveFragment, "").replaceFirst(regex.quoteRemove, "");
		}
		return result;
	}
	
	private void getRelative() {
	
		String str = "";
		Pattern p = Pattern.compile(regex.regexFragmentPartA);
		Matcher m = p.matcher(this.bufHtml);
		if (m.find()) {
			str = m.group();
		}
		if (str.equals("")) {
			p = Pattern.compile(regex.regexFragmentPartB);
			m = p.matcher(this.bufHtml);
			if (m.find()) {
				str = m.group();
			}
		}
		
		this.relative = setRelative(str);
	}
}
