import java.util.LinkedList;

/**
 * mechanism of work queue
 */
public class Workqueue {

		private final int nthread;
		private final LinkedList<Runnable> queue;
		private final poolWorker[] threads;
		private boolean running;
		
		public Workqueue(int nthread) {
			
			this.running = true;
			this.nthread = nthread;
			this.queue = new LinkedList<Runnable>();
			this.threads = new poolWorker[nthread];
		
			for (int i = 0 ; i < this.nthread ; i++) {
				threads[i] = new poolWorker();
				threads[i].start();
			}
		
		}
		
		public boolean isFinished() {
			if (!queue.isEmpty()) {
				return false;
			}
			boolean isDone = true;
			for (poolWorker worker : threads) {
				if (worker.getState() != Thread.State.WAITING) {
					isDone = false;
					break;
				}
			}
			return isDone;
		}
		
		public void threadStop() {
			while (!isFinished()) {
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			running = false;
			synchronized(queue) {
				queue.notifyAll();
			}
		}
		
		public void execute(Runnable r) {
			synchronized(queue) {
				queue.add(r);
				queue.notify();
			}
		}
		
		private class poolWorker extends Thread {
			
			public void run() {
				
				Runnable r = null;
				
				while(running) {
					synchronized(queue) {
						while(queue.isEmpty()) {
							try {
								if (!running) {
									break;
								}
								queue.wait();
							} catch (InterruptedException intException) {
							}
						}
						if(!queue.isEmpty()) {
							r = (Runnable) queue.removeFirst();
						}
					}
					try {
						if (r != null) {
							r.run();
						}
					} catch (RuntimeException e) {
						e.printStackTrace();
						System.out.println("Thread Error : Run Time Exception");
					}
				}	
			}
		}
}
